@@ -723,10 +723,10 @@ class Worker(object):
             elif _is_external(task):
                 deps = None
                 status = PENDING
-                runnable = worker().retry_external_tasks
+                runnable = self._config.retry_external_tasks
                 task.trigger_event(Event.DEPENDENCY_MISSING, task)
                 logger.warning('Data for %s does not exist (yet?). The task is an '
-                               'external data depedency, so it can not be run from'
+                               'external data dependency, so it cannot be run from'
                                ' this luigi process.', task)
 
             else:
